% vms_params mode -- special mode to facilitate editing of modparams files on VMS systems.
% hb, 20-Mar-2015
%

static variable vms_params_indent = 4;
static variable vms_params_continue = 8;

CASE_SEARCH = 0;

static define vms_params_bol () {
    if (bolp()) {
        newline();
        return 1;
    }
    return 0;
}

static define vms_params_comment() {
    push_spot();
    bol();
    if (looking_at("!")) {
        pop_spot();
        newline();
        insert ("! ");
        return 1;
    }
    pop_spot();
    return 0;
}

% vms_params_last_indent should be recursive, but I don't know how to do this in .sl
% so the caller has to call again with greater up_lines
static define vms_params_last_indent(up_lines) {
    variable lcol;
    push_spot();
    if (up(up_lines)!=up_lines or bolp()) {
        pop_spot();
        return 1;
    }
    eol();
    left(1);
    if (looking_at("-")) {
        right(1);
        % lcol = vms_params_last_indent ();
        % return 0 so the caller can call this again
        lcol = 0;
        pop_spot();
        return lcol;
    }
    down_1();
    if (looking_at("!")) {
        eol();
        % lcol = vms_params_last_indent ();
        % return 0 so the caller can call this again
        lcol = 0;
        pop_spot();
        return lcol;
    }
    skip_white();
    lcol = what_column();
    pop_spot();
    return lcol;
}

static define vms_params_this_indent() {
    variable lcol;
    if (bolp())
        return 1;
    push_spot();
    bol();
    if (looking_at("!")) {
        pop_spot();
        return 1;
    }
    skip_white();
    lcol = what_column();
    pop_spot();
    return lcol;
}

static define vms_params_cont() {
    left(1);
    if (looking_at("-")) {
        variable col, ins;
        right(1);
        col = vms_params_this_indent();
        if (col<=3)
            col = 1;
        ins = col-1+vms_params_continue;
        push_spot();
        if (up_1() and not bolp() and 1==left(1) and looking_at("-"))
            ins = col-1;
        pop_spot();
        newline();
        insert_spaces(ins);
        return 1;
    }
    right(1);
    return 0;
}

static define vms_params_normal () {
    variable col, i;
    col = vms_params_this_indent();
    push_spot();
    i = 1;
    if (up_1() and not bolp() and 1==left(1) and looking_at("-")) {
        down_1();
        col = vms_params_last_indent(i);
        while (col==0) {
                ++i;
            col=vms_params_last_indent(i);
        }
    }
    pop_spot();
    newline();
    insert_spaces(col-1);
}

define vms_params_newline () {
    trim();
    % at the begin of line just insert a new line
        if (vms_params_bol())
        return;
    % continue a full line comment (!...)
    if (vms_params_comment())
        return;
    if (vms_params_cont())
        return;
    vms_params_normal();
}

create_syntax_table("vms_params");

#ifdef HAS_DFA_SYNTAX
%%% DFA_CACHE_BEGIN %%%
private define setup_dfa_callback (name) {
%    dfa_enable_highlight_cache("vms_params.dfa", name);
    dfa_define_highlight_rule("!.*$", "comment", name);
    dfa_define_highlight_rule("\"[^\"]*\"", "string", name);
    dfa_define_highlight_rule("[aA][dD][dD]_", "Qpreprocess", name);
    dfa_define_highlight_rule("[mM][aA][xX]_", "Qpreprocess", name);
    dfa_define_highlight_rule("[mM][iI][nN]_", "Qpreprocess", name);
    dfa_define_highlight_rule("[pP][aA][gG][eE][fF][iI][lL][eE][0-9]+_[nN][aA][mM][eE]", "preprocess", name);
    dfa_define_highlight_rule("[pP][aA][gG][eE][fF][iI][lL][eE][0-9]+_[sS][iI][zZ][eE]", "preprocess", name);
    dfa_define_highlight_rule("[sS][wW][aA][pP][fF][iI][lL][eE][0-9]+_[nN][aA][mM][eE]", "preprocess", name);
    dfa_define_highlight_rule("[sS][wW][aA][pP][fF][iI][lL][eE][0-9]+_[sS][iI][zZ][eE]", "preprocess", name);
    dfa_define_highlight_rule("agen\\$include_params", "preprocess", name);
    dfa_define_highlight_rule("[a-zA-Z][a-zA-Z0-9_;:\\[\\]\\<\\>\\.\\$\\-]*", "Knormal", name);
    dfa_define_highlight_rule("[0-9]+", "number", name);
    dfa_define_highlight_rule("%[oO][0-7]+", "number", name);
    dfa_define_highlight_rule("%[dD][0-9]+", "number", name);
    dfa_define_highlight_rule("%[xX][0-9a-fA-F]+", "number", name);
    dfa_build_highlight_table(name);
}
dfa_set_init_callback (&setup_dfa_callback, "vms_params");
%%% DFA_CACHE_END %%%
#endif
% ALL params -> usually "green"
() = define_keywords_n ("vms_params",
			"user3user4voteswsdecwsincwsmax",
			5, 1);
() = define_keywords_n ("vms_params",
			"awsmindefgiddefpridefuidgalaxymaxbufpfrathpfratlus"+
			"erd1userd2",
			6, 1);
() = define_keywords_n ("vms_params",
			"awstimedumpbugfreelimgrowlimijoblimpagedynquantum"+
			"rjoblims2_sizescsnodesettimetty_buf",
			7, 1);
() = define_keywords_n ("vms_params",
			"extracpufreegoalgblpageslockrmwtlongwaitminwscnt"+
			"mountmsgnpagedynnpageradnpagevirpanopollpasanity"+
			"poweroffsavedumpscsmaxdgsysmwcnt",
			8, 1);
() = define_keywords_n ("vms_params",
			"acp_sharealloclassbalsetcntborrowlimbugreboot"+
			"clisymtblcrdenabledefquepridismoumsgdumpstyle"+
			"gblpagfillan_flagslockdirwtlockidtblmaxbobmem"+
			"maxqueprimscp_loadmvtimeoutnpagecalcpamaxport"+
			"panumpollpastdgbufpastimoutqdskvotesrms_dfmbc"+
			"rms_dfnbcscsmaxmsgshadowingtimvcfailtty_speed"+
			"vhpt_sizewlksysdsk",
			9, 1);
() = define_keywords_n ("vms_params",
			"acp_sysaccacp_windowchannelcntclass_protfile_cache"+
			"mpw_threshpfcdefaultpql_dastlmpql_dbiolmpql_dbytlm"+
			"pql_dcpulmpql_ddiolmpql_denqlmpql_dfillmpql_dprclm"+
			"pql_dtqelmpql_mastlmpql_mbiolmpql_mbytlmpql_mcpulm"+
			"pql_mdiolmpql_menqlmpql_mfillmpql_mprclmpql_mtqelm"+
			"reshashtblscsbuffcntscsrespcntsmci_flagssmci_ports"+
			"ssi_enablestartup_p1startup_p2startup_p3startup_p4"+
			"startup_p5startup_p6startup_p7startup_p8tcpip_load"+
			"tmscp_loadtty_paritytty_rspeedvaxcluster",
			10, 1);
() = define_keywords_n ("vms_params",
			"acp_maxreadacp_worksetacp_xqp_resarb_support"+
			"defmbxmxmsgdelprc_exitdisk_quorumdormantwait"+
			"fclan_framegblsectionsgh_res_codegh_res_data"+
			"glx_shm_regkstackpageslckmgr_modelgi_brk_lim"+
			"lgi_brk_tmolgi_hid_timlgi_pwd_tmolnmphashtbl"+
			"lnmshashtblmaxsysgroupmpw_hilimitmpw_iolimit"+
			"mpw_lolimitmscp_buffermultithreadnpag_gentle"+
			"procsectcntrad_supportscsflowcushscssystemid"+
			"swpoutpgcnttty_defchartty_dmasizetty_timeout"+
			"wbm_msg_int",
			11, 1);
() = define_keywords_n ("vms_params",
			"acp_baseprio"+
			"acp_dircache"+
			"acp_extcache"+
			"acp_extlimit"+
			"acp_fidcache"+
			"acp_hdrcache"+
			"acp_mapcache"+
			"acp_multiple"+
			"acp_quocache"+
			"acp_swapflgs"+
			"defmbxbufquo"+
			"dr_unit_base"+
			"gh_exec_code"+
			"gh_exec_data"+
			"gh_rsrvpgcnt"+
			"glx_inst_tmo"+
			"ieee_address"+
			"imgreg_pages"+
			"lckmgr_cpuid"+
			"lgi_brk_term"+
			"lgi_callouts"+
			"mmg_ctlflags"+
			"mscp_cmd_tmo"+
			"mscp_credits"+
			"net_callouts"+
			"npag_bap_max"+
			"npag_bap_min"+
			"pql_djtquota"+
			"pql_dwsquota"+
			"pql_mjtquota"+
			"pql_mwsquota"+
			"qdskinterval"+
			"rms_dfmbfidx"+
			"rms_dfmbfrel"+
			"rms_dfmbfsdk"+
			"rms_dfmbfsmt"+
			"rms_dfmbfsur"+
			"rms_fileprot"+
			"rms_path_tmo"+
			"rms_prologue"+
			"scssystemidh"+
			"smp_spinwait"+
			"system_check"+
			"tty_altalarm"+
			"tty_altypahd"+
			"tty_autochar"+
			"tty_defchar2"+
			"tty_defchar3"+
			"tty_dialtype"+
			"tty_silotime"+
			"tty_typahdsz"+
			"uafalternate"+
			"zero_list_hi",
			12, 1);
() = define_keywords_n ("vms_params",
			"acp_datacheck"+
			"acp_rebldsysd"+
			"acp_writeback"+
			"bugcheckfatal"+
			"dbgtk_scratch"+
			"deadlock_wait"+
			"device_naming"+
			"gb_defpercent"+
			"ieee_addressh"+
			"lgi_retry_lim"+
			"lgi_retry_tmo"+
			"maxprocesscnt"+
			"mpw_waitlimit"+
			"niscs_use_lan"+
			"niscs_use_udp"+
			"npag_interval"+
			"pql_dwsextent"+
			"pql_mwsextent"+
			"recnxinterval"+
			"rms_conpolicy"+
			"shadow_enable"+
			"tty_classname"+
			"tty_scandelta"+
			"vcc_max_cache"+
			"vcc_max_locks"+
			"vcc_readahead"+
			"wbm_msg_lower"+
			"wbm_msg_upper"+
			"wbm_opcom_lvl"+
			"window_system",
			13, 1);
() = define_keywords_n ("vms_params",
			"acp_dindxcache"+
			"cpu_power_mgmt"+
			"erlbufferpages"+
			"execstackpages"+
			"expected_votes"+
			"gb_cacheallmax"+
			"gh_res_code_s2"+
			"mpw_stackpages"+
			"mpw_wrtcluster"+
			"mscp_serve_all"+
			"niscs_udp_port"+
			"npag_ring_size"+
			"paged_lal_size"+
			"papollinterval"+
			"papoolinterval"+
			"pql_dpgflquota"+
			"pql_dwsdefault"+
			"pql_mpgflquota"+
			"pql_mwsdefault"+
			"prcpolinterval"+
			"sch_hard_offld"+
			"sch_soft_offld"+
			"shadow_mbr_tmo"+
			"shadow_rec_dly"+
			"shadow_site_id"+
			"shadow_sys_tmo"+
			"smp_cpu_bitmap"+
			"smp_sanity_cnt"+
			"tape_alloclass"+
			"tape_mvtimeout"+
			"timepromptwait",
			14, 1);
() = define_keywords_n ("vms_params",
			"auto_dlight_sav"+
			"cluster_credits"+
			"cpu_power_thrsh"+
			"cwcreprc_enable"+
			"erlbufferpag_s2"+
			"errorlogbuff_s2"+
			"errorlogbuffers"+
			"io_prcpu_bitmap"+
			"lgi_brk_disuser"+
			"load_pwd_policy"+
			"mpw_lowaitlimit"+
			"multiprocessing"+
			"niscs_conv_boot"+
			"niscs_load_pea0"+
			"niscs_max_pktsz"+
			"niscs_port_serv"+
			"niscs_udp_pktsz"+
			"npag_aggressive"+
			"npag_bap_max_pa"+
			"npag_bap_min_pa"+
			"rms_extend_size"+
			"rms_seqfile_wbh"+
			"security_policy"+
			"shadow_hbmm_rtc"+
			"shadow_max_copy"+
			"shadow_max_unit"+
			"shadow_psm_rdly"+
			"shadow_sys_disk"+
			"shadow_sys_unit"+
			"shadow_sys_wait"+
			"smp_lngspinwait"+
			"ssio_sync_intvl"+
			"tmscp_serve_all"+
			"vcc_max_io_size"+
			"vcc_write_delay"+
			"vcc_writebehind",
			15, 1);
% SPECIAL params -> usually "red"
() = define_keywords_n ("vms_params",
			"pe1pe2pe3pe4"+
			"pe5pe6",
			3, 0);
() = define_keywords_n ("vms_params",
			"iota"+
			"vms5vms6vms7vms8",
			4, 0);
() = define_keywords_n ("vms_params",
			"rmsd1rmsd2rmsd3rmsd4rmsd5rmsd6rmsd7vmsd1vmsd2vmsd3"+
			"vmsd4",
			5, 0);
() = define_keywords_n ("vms_params",
			"ktk_d1ktk_d2ktk_d3ktk_d4ktk_d5ktk_d6ktk_d7ktk_d8"+
			"ktk_d9syspfcwbm_d1",
			6, 0);
() = define_keywords_n ("vms_params",
			"dnvosi1f64ctl1f64ctl2"+
			"jobctldktk_d10ktk_d11ktk_d12pixscanswpfailswprate"+
			"xqpctl2xqpctl4xqpctl6xqpctl8",
			7, 0);
() = define_keywords_n ("vms_params",
			"ctlpages"+
			"f64ctld3f64ctld4imgiocntminprprimpdev_d1mpdev_d2"+
			"mpdev_d3mpdev_d4mpw_priopiopages"+
			"resalloc"+
			"swis_logswp_prio"+
			"vcc_rsvdxqpctld1xqpctld3xqpctld5xqpctld7",
			8, 0);
() = define_keywords_n ("vms_params",
			"ctlimglimfast_path"+
			"lockretrynocluster"+
			"nopgflswppagtblpfc"+
			"poolcheckpseudoloa"+
			"rms_dflrl"+
			"rsvd_io_1rsvd_io_2"+
			"s0_paging"+
			"shadow_d1shadow_d2shadow_d3"+
			"shadow_d4shadow_d5"+
			"ssinhibit"+
			"tbskipwsl"+
			"vcc_flags"+
			"wpre_size",
			9, 0);
() = define_keywords_n ("vms_params",
			"poolpaging"+
			"pu_optionsrsrvpagcntrsvd_clu_1rsvd_clu_2rsvd_lan_1"+
			"rsvd_lan_2sas_namingtesting123wptte_size",
			10, 0);
() = define_keywords_n ("vms_params",
			"breakpointscrd_controllckmgr_rsvdmaxclasspri"+
			"minclasspriplatf_spt_1platf_spt_2platf_spt_3platf_spt_4"+
			"rsvd_exec_1rsvd_exec_2sched_flags"+
			"tty_defportvcc_maxsizewritablesys",
			11, 0);
() = define_keywords_n ("vms_params",
			"dcl_ctlflags"+
			"dlckextrastk"+
			"mpdev_enablempdev_pollermpdev_remote"+
			"mvsupmsg_numnoautoconfigplatf_spt_d1platf_spt_d2"+
			"platf_spt_d3platf_spt_d4rsvd_secur_1rsvd_secur_2"+
			"smp_tick_cnt"+
			"time_controlvcc_pagesize",
			12, 0);
() = define_keywords_n ("vms_params",
			"affinity_skipaffinity_time"+
			"cpu_threading"+
			"phymem_configrms_heuristic",
			13, 0);
() = define_keywords_n ("vms_params",
			"clock_intervalfile_cache_maxfile_cache_min"+
			"mc_services_p0mc_services_p1mc_services_p2"+
			"mc_services_p3mc_services_p4mc_services_p5"+
			"mc_services_p6mc_services_p7mc_services_p8"+
			"mc_services_p9mvsupmsg_intvlpersistent_res"+
			"scsicluster_p1scsicluster_p2scsicluster_p3"+
			"scsicluster_p4"+
			"sysser_loggingwritesysparams"+
			"xqp_alloc_blks",
			14, 0);
() = define_keywords_n ("vms_params",
			"conceal_devices"+
			"disable_upcalls"+
			"fast_path_ports"+
			"fibre_scsi_rsv1"+
			"load_sys_images"+
			"mpdev_afb_intvl"+
			"mpdev_lcretries"+
			"pfn_color_countphysical_memorypriority_offset"+
			"scsi_error_poll",
			15, 0);

set_color ("keyword2", get_color ("preprocess"));
() = define_keywords_n ("vms_params", "bootnodedumpfilepagefileswapfile", 8, 2);
() = define_keywords_n ("vms_params", "num_nodes", 9, 2);
() = define_keywords_n ("vms_params", "errlogdump", 10, 2);
() = define_keywords_n ("vms_params", "interconnect", 12, 2);
() = define_keywords_n ("vms_params", "num_etheradapt", 14, 2);
() = define_keywords_n ("vms_params", "dumpfile_device", 15, 2);
() = define_keywords_n ("vms_params", "agen$include_params", 19, 2);

define_syntax ("([<", ")]>", '(', "vms_params");

!if (keymap_p ("VMS_PARAMS")) {
    make_keymap ("VMS_PARAMS");
    definekey ("vms_params_newline", "^M", "VMS_PARAMS");
    definekey ("self_insert_cmd", "^I", "VMS_PARAMS");
    definekey ("self_insert_cmd", "\"", "VMS_PARAMS");
    definekey ("self_insert_cmd", "'", "VMS_PARAMS");
}

define vms_params_mode () {
    set_syntax_flags ("vms_params",0x81);
    use_syntax_table("vms_params");
    use_dfa_syntax(1);
   
    use_keymap ("VMS_PARAMS");
    set_mode ("VMS_PARAMS", 4);
    run_mode_hooks("vms_params_mode_hook");
}
